@@ -19,8 +19,8 @@
 #include "darkstat.h"
 #include "acct.h"
 #include "conv.h"
+#include "daylog.h"
 #include "err.h"
-#include "graph_db.h"
 #include "hosts_db.h"
 #include "localip.h"
 
@@ -129,10 +129,14 @@ acct_for(const pktsummary *sm)
          dir_in = 1;
    }
 
-   if (dir_out)
+   if (dir_out) {
+      daylog_acct((uint64_t)sm->len, GRAPH_OUT);
       graph_acct((uint64_t)sm->len, GRAPH_OUT);
-   if (dir_in)
+   }
+   if (dir_in) {
+      daylog_acct((uint64_t)sm->len, GRAPH_IN);
       graph_acct((uint64_t)sm->len, GRAPH_IN);
+   }
 
    /* Protocols. */
    ps = host_get_ip_proto(hs, sm->proto);
