@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * darkstat.h: general macros
  */
@@ -34,7 +34,4 @@
 # define min(a,b) ((a) < (b) ? (a) : (b))
 #endif
 
-#define verbosef warnx
-#define dverbosef (void) /* disabled verbosef */
-
 /* vim:set ts=3 sw=3 tw=78 expandtab: */
