@@ -7,8 +7,12 @@
  * GNU General Public License version 2. (see COPYING.GPL)
  */
 
+#include <sys/types.h>
+#include <netinet/in.h> /* for in_addr_t (db.h needs it) */
+
 #include "conv.h"
 #include "darkstat.h"
+#include "db.h"
 #include "acct.h"
 #include "err.h"
 #include "str.h"
@@ -28,13 +32,14 @@ struct graph {
    unsigned int offset; /* i.e. seconds start at 0, days start at 1 */
    unsigned int pos, num_bars;
    const char *unit;
+   unsigned int bar_secs; /* one bar represents <n> seconds */
 };
 
 static struct graph
-   graph_secs = {NULL, NULL, 0, 0, 60, "seconds"},
-   graph_mins = {NULL, NULL, 0, 0, 60, "minutes"},
-   graph_hrs  = {NULL, NULL, 0, 0, 24, "hours"},
-   graph_days = {NULL, NULL, 1, 0, 31, "days"};
+   graph_secs = {NULL, NULL, 0, 0, 60, "seconds", 1},
+   graph_mins = {NULL, NULL, 0, 0, 60, "minutes", 60},
+   graph_hrs  = {NULL, NULL, 0, 0, 24, "hours",   3600},
+   graph_days = {NULL, NULL, 1, 0, 31, "days",    86400};
 
 static struct graph *graph_db[] = {
    &graph_secs, &graph_mins, &graph_hrs, &graph_days
@@ -42,17 +47,34 @@ static struct graph *graph_db[] = {
 
 static unsigned int graph_db_size = sizeof(graph_db)/sizeof(*graph_db);
 
-static time_t start_time = 0;
+static time_t start_time, last_time;
 
 void
 graph_init(void)
 {
    unsigned int i;
    for (i=0; i<graph_db_size; i++) {
-      graph_db[i]->in  = xcalloc(sizeof(uint64_t), graph_db[i]->num_bars);
-      graph_db[i]->out = xcalloc(sizeof(uint64_t), graph_db[i]->num_bars);
+      graph_db[i]->in  = xmalloc(sizeof(uint64_t) * graph_db[i]->num_bars);
+      graph_db[i]->out = xmalloc(sizeof(uint64_t) * graph_db[i]->num_bars);
    }
    start_time = time(NULL);
+   graph_reset();
+}
+
+static void
+zero_graph(struct graph *g)
+{
+   memset(g->in,  0, sizeof(uint64_t) * g->num_bars);
+   memset(g->out, 0, sizeof(uint64_t) * g->num_bars);
+}
+
+void
+graph_reset(void)
+{
+   unsigned int i;
+   for (i=0; i<graph_db_size; i++)
+      zero_graph(graph_db[i]);
+   last_time = 0;
 }
 
 void
@@ -70,20 +92,13 @@ graph_acct(uint64_t amount, enum graph_dir dir)
 {
    unsigned int i;
    for (i=0; i<graph_db_size; i++)
-   switch (dir) {
-   case GRAPH_IN:
-      graph_db[i]->in[ graph_db[i]->pos ] += amount;
-      break;
-   case GRAPH_OUT:
-      graph_db[i]->out[ graph_db[i]->pos ] += amount;
-      break;
-   default:
-      errx(1, "unknown graph_dir in graph_acct: %d", dir);
-   }
+    switch (dir) {
+     case GRAPH_IN:  graph_db[i]->in[  graph_db[i]->pos ] += amount; break;
+     case GRAPH_OUT: graph_db[i]->out[ graph_db[i]->pos ] += amount; break;
+     default: errx(1, "unknown graph_dir in graph_acct: %d", dir);
+    }
 }
 
-static time_t last_time = 0;
-
 /* Advance a graph: advance the pos, zeroing out bars as we move. */
 static void
 advance(struct graph *g, const unsigned int pos)
@@ -165,8 +180,9 @@ graph_resync(const time_t new_time)
 void
 graph_rotate(void)
 {
-   time_t t;
+   time_t t, td;
    struct tm *tm;
+   unsigned int i;
 
    t = now;
 
@@ -195,16 +211,94 @@ graph_rotate(void)
    }
 
    /* else, normal rotation */
+   td = t - last_time;
    last_time = t;
    tm = localtime(&t);
    if (tm->tm_sec == 60)
       tm->tm_sec = 59; /* mis-handle leap seconds */
+
+   /* zero out graphs which have been completely rotated through */
+   for (i=0; i<graph_db_size; i++)
+      if (td >= (int)(graph_db[i]->num_bars * graph_db[i]->bar_secs))
+         zero_graph(graph_db[i]);
+
+   /* advance the current position, zeroing up to it */
    advance(&graph_secs, tm->tm_sec);
    advance(&graph_mins, tm->tm_min);
    advance(&graph_hrs, tm->tm_hour);
    advance(&graph_days, tm->tm_mday - 1);
 }
 
+/* ---------------------------------------------------------------------------
+ * Database Import: Grab graphs from a file provided by the caller.
+ *
+ * This function will retrieve the data sans the header.  We expect the caller
+ * to have validated the header of the segment, and left the file position at
+ * the start of the data.
+ */
+int
+graph_import(const int fd)
+{
+   uint64_t last;
+   unsigned int i, j;
+
+   if (!read64(fd, &last)) return 0;
+   last_time = (time_t)last;
+
+   for (i=0; i<graph_db_size; i++) {
+      unsigned char num_bars, pos;
+      unsigned int filepos = xtell(fd);
+
+      if (!read8(fd, &num_bars)) return 0;
+      if (!read8(fd, &pos)) return 0;
+
+      verbosef("at file pos %u, importing graph with %u bars",
+         filepos, (unsigned int)num_bars);
+
+      if (pos >= num_bars) {
+         warn("pos is %u, should be < num_bars which is %u",
+            (unsigned int)pos, (unsigned int)num_bars);
+         return 0;
+      }
+
+      if (graph_db[i]->num_bars != num_bars) {
+         warn("num_bars is %u, expecting %u",
+            (unsigned int)num_bars, graph_db[i]->num_bars);
+         return 0;
+      }
+
+      graph_db[i]->pos = pos;
+      for (j=0; j<num_bars; j++) {
+         if (!read64(fd, &(graph_db[i]->in[j]))) return 0;
+         if (!read64(fd, &(graph_db[i]->out[j]))) return 0;
+      }
+   }
+
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Database Export: Dump hosts_db into a file provided by the caller.
+ * The caller is responsible for writing out the header first.
+ */
+int
+graph_export(const int fd)
+{
+   unsigned int i, j;
+
+   if (!write64(fd, (uint64_t)last_time)) return 0;
+   for (i=0; i<graph_db_size; i++) {
+      if (!write8(fd, graph_db[i]->num_bars)) return 0;
+      if (!write8(fd, graph_db[i]->pos)) return 0;
+
+      for (j=0; j<graph_db[i]->num_bars; j++) {
+         if (!write64(fd, graph_db[i]->in[j])) return 0;
+         if (!write64(fd, graph_db[i]->out[j])) return 0;
+      }
+   }
+   return 1;
+}
+
 /* ---------------------------------------------------------------------------
  * Format a length of time in seconds to something more readable.
  */
@@ -280,7 +374,7 @@ html_front_page(void)
 
    str_append(buf, "<div>\n");
    for (i=0; i<graph_db_size; i++) {
-      str_appendf(buf,"<div class=\"outergraph\" id=\"g%d\">%s</div>\n",
+      str_appendf(buf,"<div class=\"outergraph\" id=\"g%u\">%s</div>\n",
          i, (i==0)?"Graphs require JavaScript.":"");
       /* two above, break, two below */
       if (i == 1)
@@ -304,9 +398,13 @@ html_front_page(void)
 
    for (i=0; i<graph_db_size; i++)
       str_appendf(buf,
-         " {id:\"g%d\", name:\"%s\", title:\"last %d %s\"}%s\n",
+         " {id:\"g%u\", "
+           "name:\"%s\", "
+           "title:\"last %u %s\", "
+           "bar_secs:%u"
+           "}%s\n",
          i, graph_db[i]->unit, graph_db[i]->num_bars, graph_db[i]->unit,
-         (i < graph_db_size-1) ? "," : "");
+         graph_db[i]->bar_secs, (i < graph_db_size-1) ? "," : "");
       /* trailing comma breaks IE, it makes the array one element bigger */
 
    str_append(buf,
