@@ -13,6 +13,7 @@
 #include "dns.h"
 #include "err.h"
 #include "hosts_db.h"
+#include "db.h"
 #include "html.h"
 #include "ncache.h"
 
@@ -22,6 +23,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h> /* memset(), strcmp() */
+#include <unistd.h>
 
 int show_mac_addrs = 0;
 
@@ -117,6 +119,12 @@ hash_func_short(const struct hashtable *h, const void *key)
    return (CASTKEY(uint16_t) * h->coeff);
 }
 
+static uint32_t
+hash_func_byte(const struct hashtable *h, const void *key)
+{
+   return (CASTKEY(uint8_t) * h->coeff);
+}
+
 /* ---------------------------------------------------------------------------
  * key_func collection
  */
@@ -170,7 +178,7 @@ find_func_port_udp(const struct bucket *b, const void *key)
 static int
 find_func_ip_proto(const struct bucket *b, const void *key)
 {
-   return (b->u.ip_proto.proto == CASTKEY(uint16_t));
+   return (b->u.ip_proto.proto == CASTKEY(uint8_t));
 }
 
 /* ---------------------------------------------------------------------------
@@ -220,7 +228,7 @@ static struct bucket *
 make_func_ip_proto(const void *key)
 {
    MAKE_BUCKET(b, p, ip_proto);
-   p->proto = CASTKEY(uint16_t);
+   p->proto = CASTKEY(uint8_t);
    return (b);
 }
 
@@ -647,6 +655,28 @@ hosts_db_reduce(void)
    hashtable_rehash(hosts_db, hosts_db->bits); /* needed? */
 }
 
+/* ---------------------------------------------------------------------------
+ * Reset hosts_db to empty.
+ */
+void
+hosts_db_reset(void)
+{
+   unsigned int i;
+
+   for (i=0; i<hosts_db->size; i++) {
+      struct bucket *next, *b = hosts_db->table[i];
+      while (b != NULL) {
+         next = b->next;
+         host_free(&(b->u.host));
+         free(b);
+         b = next;
+      }
+      hosts_db->table[i] = NULL;
+   }
+   verbosef("hosts_db reset to empty, freed %u hosts", hosts_db->count);
+   hosts_db->count = 0;
+}
+
 /* ---------------------------------------------------------------------------
  * Deallocate hosts_db.
  */
@@ -705,13 +735,13 @@ host_get_port_udp(struct bucket *host, const uint16_t port)
  * Find or create an ip_proto inside a host.
  */
 struct bucket *
-host_get_ip_proto(struct bucket *host, const uint16_t proto)
+host_get_ip_proto(struct bucket *host, const uint8_t proto)
 {
    struct host *h = &host->u.host;
    assert(h != NULL);
    if (h->ip_protos == NULL)
       h->ip_protos = hashtable_make(PROTO_BITS,
-         hash_func_short, key_func_ip_proto,
+         hash_func_byte, key_func_ip_proto,
          find_func_ip_proto, make_func_ip_proto,
          format_cols_ip_proto, format_row_ip_proto);
    return (hashtable_find_or_insert(h->ip_protos, &proto));
@@ -1059,4 +1089,340 @@ html_proto_stats(const char *ip)
    return html_hashtable_stats(h->u.host.ip_protos);
 }
 
+/* ---------------------------------------------------------------------------
+ * Database import and export code:
+ * Initially written and contributed by Ben stewart.
+ * copyright (c) 2007 Ben Stewart, Emil Mikulic.
+ */
+static int hosts_db_export_ip(const struct hashtable *h, const int fd);
+static int hosts_db_export_tcp(const struct hashtable *h, const int fd);
+static int hosts_db_export_udp(const struct hashtable *h, const int fd);
+
+static const char export_proto_ip  = 'P';
+static const char export_proto_tcp = 'T';
+static const char export_proto_udp = 'U';
+
+static const unsigned char export_tag_host_ver1[] = {'H', 'S', 'T', 0x01};
+
+/* ---------------------------------------------------------------------------
+ * Load a host's ip_proto table from a file.
+ * Returns 0 on failure, 1 on success.
+ */
+static int
+hosts_db_import_ip(const int fd, struct bucket *host)
+{
+   uint8_t count, i;
+
+   if (!expect8(fd, export_proto_ip)) return 0;
+   if (!read8(fd, &count)) return 0;
+
+   for (i=0; i<count; i++) {
+      struct bucket *b;
+      uint8_t proto;
+      uint64_t in, out;
+
+      if (!read8(fd, &proto)) return 0;
+      if (!read64(fd, &in)) return 0;
+      if (!read64(fd, &out)) return 0;
+
+      /* Store data */
+      b = host_get_ip_proto(host, proto);
+      b->in = in;
+      b->out = out;
+      b->total = in + out;
+      assert(b->u.ip_proto.proto == proto); /* should be done by make fn */
+   }
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Load a host's port_tcp table from a file.
+ * Returns 0 on failure, 1 on success.
+ */
+static int
+hosts_db_import_tcp(const int fd, struct bucket *host)
+{
+   uint16_t count, i;
+
+   if (!expect8(fd, export_proto_tcp)) return 0;
+   if (!read16(fd, &count)) return 0;
+
+   for (i=0; i<count; i++) {
+      struct bucket *b;
+      uint16_t port;
+      uint64_t in, out, syn;
+
+      if (!read16(fd, &port)) return 0;
+      if (!read64(fd, &syn)) return 0;
+      if (!read64(fd, &in)) return 0;
+      if (!read64(fd, &out)) return 0;
+
+      /* Store data */
+      b = host_get_port_tcp(host, port);
+      b->in = in;
+      b->out = out;
+      b->total = in + out;
+      assert(b->u.port_tcp.port == port); /* done by make_func_port_tcp */
+      b->u.port_tcp.syn = syn;
+   }
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Load a host's port_tcp table from a file.
+ * Returns 0 on failure, 1 on success.
+ */
+static int
+hosts_db_import_udp(const int fd, struct bucket *host)
+{
+   uint16_t count, i;
+
+   if (!expect8(fd, export_proto_udp)) return 0;
+   if (!read16(fd, &count)) return 0;
+
+   for (i=0; i<count; i++) {
+      struct bucket *b;
+      uint16_t port;
+      uint64_t in, out;
+
+      if (!read16(fd, &port)) return 0;
+      if (!read64(fd, &in)) return 0;
+      if (!read64(fd, &out)) return 0;
+
+      /* Store data */
+      b = host_get_port_udp(host, port);
+      b->in = in;
+      b->out = out;
+      b->total = in + out;
+      assert(b->u.port_udp.port == port); /* done by make_func */
+   }
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Load all hosts from a file.
+ * Returns 0 on failure, 1 on success.
+ */
+static int
+hosts_db_import_host(const int fd)
+{
+   struct bucket *host;
+   in_addr_t addr;
+   uint8_t hostname_len;
+   uint64_t in, out;
+   unsigned int pos = xtell(fd);
+
+   if (!read_file_header(fd, export_tag_host_ver1)) return 0;
+
+   /* HOST ENTRY */
+   if (!readaddr(fd, &addr)) return 0;
+   verbosef("at file pos %u, importing host %s", pos, ip_to_str(addr));
+   host = host_get(addr);
+   assert(host->u.host.ip == addr); /* make fn? */
+
+   assert(sizeof(host->u.host.mac_addr) == 6);
+   if (!readn(fd, host->u.host.mac_addr, sizeof(host->u.host.mac_addr)))
+      return 0;
+
+   /* HOSTNAME */
+   assert(host->u.host.dns == NULL); /* make fn? */
+   if (!read8(fd, &hostname_len)) return 0;
+   if (hostname_len > 0) {
+      host->u.host.dns = xmalloc(hostname_len + 1);
+      host->u.host.dns[0] = '\0';
+
+      /* At this point, the hostname is attached to a host which is in our
+       * hosts_db, so if we bail out due to an import error, this pointer
+       * isn't lost and leaked, it can be cleaned up in hosts_db_{free,reset}
+       */
+
+      if (!readn(fd, host->u.host.dns, hostname_len)) return 0;
+      host->u.host.dns[hostname_len] = '\0';
+   }
+
+   if (!read64(fd, &in)) return 0;
+   if (!read64(fd, &out)) return 0;
+
+   host->in = in;
+   host->out = out;
+   host->total = in + out;
+
+   /* Host's port and proto subtables: */
+   if (!hosts_db_import_ip(fd, host)) return 0;
+   if (!hosts_db_import_tcp(fd, host)) return 0;
+   if (!hosts_db_import_udp(fd, host)) return 0;
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Database Import: Grab hosts_db from a file provided by the caller.
+ *
+ * This function will retrieve the data sans the header.  We expect the caller
+ * to have validated the header of the hosts_db segment, and left the file
+ * sitting at the start of the data.
+ */
+int hosts_db_import(const int fd)
+{
+   uint32_t host_count, i;
+
+   if (!read32(fd, &host_count)) return 0;
+
+   for (i=0; i<host_count; i++)
+      if (!hosts_db_import_host(fd)) return 0;
+
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Database Export: Dump hosts_db into a file provided by the caller.
+ * The caller is responsible for writing out export_tag_hosts_ver1 first.
+ */
+int hosts_db_export(const int fd)
+{
+   uint32_t i;
+   struct bucket *b;
+
+   if (!write32(fd, hosts_db->count)) return 0;
+
+   for (i = 0; i<hosts_db->size; i++)
+   for (b = hosts_db->table[i]; b != NULL; b = b->next) {
+      /* For each host: */
+
+      if (!writen(fd, export_tag_host_ver1, sizeof(export_tag_host_ver1)))
+         return 0;
+
+      /* HOST ENTRY */
+      if (!writeaddr(fd, b->u.host.ip)) return 0;
+
+      /* MACADDR */
+      assert(sizeof(b->u.host.mac_addr) == 6);
+      if (!writen(fd, b->u.host.mac_addr, sizeof(b->u.host.mac_addr)))
+         return 0;
+
+      /* HOSTNAME */
+      if (b->u.host.dns == NULL) {
+         if (!write8(fd, 0)) return 0;
+      } else {
+         int dnslen = strlen(b->u.host.dns);
+
+         if (dnslen > 255) {
+           warnx("found a very long hostname: \"%s\"\n"
+              "wasn't expecting one longer than 255 chars (this one is %d)",
+              b->u.host.dns, dnslen);
+           dnslen = 255;
+         }
+
+         if (!write8(fd, (uint8_t)dnslen)) return 0;
+         if (!writen(fd, b->u.host.dns, dnslen)) return 0;
+      }
+
+      if (!write64(fd, b->in)) return 0;
+      if (!write64(fd, b->out)) return 0;
+
+      if (!hosts_db_export_ip(b->u.host.ip_protos, fd)) return 0;
+      if (!hosts_db_export_tcp(b->u.host.ports_tcp, fd)) return 0;
+      if (!hosts_db_export_udp(b->u.host.ports_udp, fd)) return 0;
+   }
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Dump the ip_proto table of a host.
+ */
+static int
+hosts_db_export_ip(const struct hashtable *h, const int fd)
+{
+   uint32_t i, written = 0;
+   struct bucket *b;
+
+   /* IP DATA */
+   if (!write8(fd, export_proto_ip)) return 0;
+
+   /* If no data, write a IP Proto count of 0 and we're done. */
+   if (h == NULL) {
+      if (!write8(fd, 0)) return 0;
+      return 1;
+   }
+
+   assert(h->count < 256);
+   if (!write8(fd, (uint8_t)h->count)) return 0;
+
+   for (i = 0; i<h->size; i++)
+   for (b = h->table[i]; b != NULL; b = b->next) {
+      /* For each ip_proto bucket: */
+
+      if (!write8(fd, b->u.ip_proto.proto)) return 0;
+      if (!write64(fd, b->in)) return 0;
+      if (!write64(fd, b->out)) return 0;
+      written++;
+   }
+   assert(written == h->count);
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Dump the port_tcp table of a host.
+ */
+static int
+hosts_db_export_tcp(const struct hashtable *h, const int fd)
+{
+   struct bucket *b;
+   uint32_t i, written = 0;
+
+   /* TCP DATA */
+   if (!write8(fd, export_proto_tcp)) return 0;
+
+   /* If no data, write a count of 0 and we're done. */
+   if (h == NULL) {
+      if (!write16(fd, 0)) return 0;
+      return 1;
+   }
+
+   assert(h->count < 65536);
+   if (!write16(fd, (uint16_t)h->count)) return 0;
+
+   for (i = 0; i<h->size; i++)
+   for (b = h->table[i]; b != NULL; b = b->next) {
+      if (!write16(fd, b->u.port_tcp.port)) return 0;
+      if (!write64(fd, b->u.port_tcp.syn)) return 0;
+      if (!write64(fd, b->in)) return 0;
+      if (!write64(fd, b->out)) return 0;
+      written++;
+   }
+   assert(written == h->count);
+   return 1;
+}
+
+/* ---------------------------------------------------------------------------
+ * Dump the port_udp table of a host.
+ */
+static int
+hosts_db_export_udp(const struct hashtable *h, const int fd)
+{
+   struct bucket *b;
+   uint32_t i, written = 0;
+
+   /* UDP DATA */
+   if (!write8(fd, export_proto_udp)) return 0;
+
+   /* If no data, write a count of 0 and we're done. */
+   if (h == NULL) {
+      if (!write16(fd, 0)) return 0;
+      return 1;
+   }
+
+   assert(h->count < 65536);
+   if (!write16(fd, (uint16_t)h->count)) return 0;
+
+   for (i = 0; i<h->size; i++)
+   for (b = h->table[i]; b != NULL; b = b->next) {
+      if (!write16(fd, b->u.port_udp.port)) return 0;
+      if (!write64(fd, b->in)) return 0;
+      if (!write64(fd, b->out)) return 0;
+      written++;
+   }
+   assert(written == h->count);
+   return 1;
+}
+
 /* vim:set ts=3 sw=3 tw=78 expandtab: */
