@@ -30,7 +30,7 @@ struct port_udp {
 };
 
 struct ip_proto {
-   uint16_t proto;
+   uint8_t proto;
 };
 
 struct bucket {
@@ -50,13 +50,16 @@ extern int show_mac_addrs;
 
 void hosts_db_init(void);
 void hosts_db_reduce(void);
+void hosts_db_reset(void);
 void hosts_db_free(void);
+int hosts_db_import(const int fd);
+int hosts_db_export(const int fd);
 
 struct bucket *host_find(const in_addr_t ip); /* can return NULL */
 struct bucket *host_get(const in_addr_t ip);
 struct bucket *host_get_port_tcp(struct bucket *host, const uint16_t port);
 struct bucket *host_get_port_udp(struct bucket *host, const uint16_t port);
-struct bucket *host_get_ip_proto(struct bucket *host, const uint16_t proto);
+struct bucket *host_get_ip_proto(struct bucket *host, const uint8_t proto);
 
 /* Web pages. */
 struct str *html_hosts(const char *uri, const char *query);
