@@ -1,5 +1,5 @@
 /* darkstat 3
- * copyright (c) 2001-2006 Emil Mikulic.
+ * copyright (c) 2001-2007 Emil Mikulic.
  *
  * localip.c: determine local IP of our capture interface
  *
@@ -8,6 +8,7 @@
  */
 
 #include "darkstat.h"
+#include "conv.h" /* for strlcpy */
 #include "decode.h" /* for ip_to_str */
 #include "err.h"
 #include "localip.h"
@@ -18,6 +19,7 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <net/if.h>
+#include <errno.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -41,8 +43,15 @@ localip_update(void)
 
    strlcpy(ifr.ifr_name, iface, IFNAMSIZ);
    ifr.ifr_addr.sa_family = AF_INET;
-   if (ioctl(tmp, SIOCGIFADDR, &ifr) == -1)
-      err(1, "can't get own IP address on interface \"%s\"", iface);
+   if (ioctl(tmp, SIOCGIFADDR, &ifr) == -1) {
+      if (errno == EADDRNOTAVAIL) {
+         /* lost IP, e.g. ifconfig eth0 delete, don't die */
+         localip = 0;
+         close(tmp);
+         return;
+      } else
+         err(1, "can't get own IP address on interface \"%s\"", iface);
+   }
    close(tmp);
    sa = ifr.ifr_addr;
 
